{

  See readme.txt for some info

}

unit Moon;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TMoonDisplayType = (mdtUseSystemTime, mdtUseCustomTime, mdtUsePhaseAngle);

  THemisphere = (heNorthern, heSouthern);

  TMoon = class(TGraphicControl)
  private
    FBackgroundColor : TColor;
    FBorderSize : integer;
    FCustomTime : string;
    FHemisphere : THemisphere;
    FMoonColor : TColor;
    FPhase : extended;
    FMoonDisplayType : TMoonDisplayType;

    procedure SetBackgroundColor(AColor : TColor);
    procedure SetBorderSize(AInteger : integer);
    procedure SetCustomTime(AString : string);
    procedure SetHemisphere(AHemisphere : THemisphere);
    procedure SetMoonColor(AColor : TColor);
    procedure SetMoonDisplayType(AMoonDisplayType : TMoonDisplayType);
    procedure SetPhase(AExtended : extended);

    procedure Paint; override;

  protected

  public
    function CalcPhase : extended;
    destructor Destroy; override;
    constructor Create(AOWner : TComponent); override;

  published
    property BackgroundColor : TColor read FBackgroundColor write SetBackgroundColor default clBlack;
    property BorderSize : integer read FBorderSize write SetBorderSize default 2;
    property CustomTime : string read FCustomTime write SetCustomTime;
    property Hemisphere : THemisphere read FHemisphere write SetHemisphere default heNorthern;
    property MoonColor : TColor read FMoonColor write SetMoonColor default clWhite;
    property Phase : extended read FPhase write SetPhase;
    property MoonDisplayType : TMoonDisplayType read FMoonDisplayType write SetMoonDisplayType;

  end;

const
  twoPiS = 2 * pi;
  AHalfPi = pi / 2;
  threeAndAHalfPi = 3 * pi / 2;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('ZP Comps', [TMoon]);
end;

constructor TMoon.Create(AOWner : TComponent);
begin
  inherited Create(AOwner);
  FBackgroundColor := clBlack;
  FBorderSize := 2;
  FCustomTime := DateTimeToStr(Now);
  FHemisphere := heNorthern;
  FMoonColor := clWhite;
  FMoonDisplayType := mdtUseSystemTime;
  FPhase := 0;
  Height := 32;
  Width := 32;
end;

destructor TMoon.Destroy;
begin
  inherited;
end;

procedure TMoon.SetMoonColor;
begin
  FMoonColor := AColor;
  Repaint;
end;

procedure TMoon.SetBackgroundColor;
begin
  FBackgroundColor := AColor;
  Repaint;
end;

procedure TMoon.SetBorderSize;
begin
  FBorderSize := AInteger;
  Repaint;
end;

procedure TMoon.SetHemisphere;
begin
  FHemisphere := AHemisphere;
  Repaint;
end;

procedure TMoon.SetCustomTime;
begin
  FMoonDisplayType := mdtUseCustomTime;
  FCustomTime := AString;
  Repaint;
end;

procedure TMoon.SetPhase;
begin
  FMoonDisplayType := mdtUsePhaseAngle;
  FPhase := AExtended;
  Repaint;
end;

procedure TMoon.SetMoonDisplayType;
begin
  FMoonDisplayType := AMoonDisplayType;
  Repaint;
end;

procedure TMoon.Paint;
var
  dx, x0, y0, rx, ry : integer;
  cPhase : extended;
begin
  inherited;

  x0 := FBorderSize;  //  starting x
  y0 := FBorderSize;  //  starting y
  rx := width - 2 * FBorderSize;    //  radius x
  ry := height - 2 * FBorderSize;   //  radius y

  Canvas.Brush.Color := FBackgroundColor;
  Canvas.Pen.Color := FBackgroundColor;
  Canvas.Rectangle(0, 0, width, height);
  Canvas.Brush.Color := FMoonColor;
  Canvas.Pen.Color := FMoonColor;
  Canvas.Ellipse(x0, y0, x0 + rx, y0 + ry);

  cPhase := CalcPhase;

  //  if cPhase > 2pi
  cPhase := cPhase - trunc(cPhase / twoPiS) * twoPiS;

  //  if the user is on the southern hemisphere
  if FHemisphere = heSouthern
    then begin
      cPhase := twoPiS - cPhase;
      cPhase := cPhase - trunc(cPhase / twoPiS) * twoPiS;
    end;

  if cPhase < AHalfPi
    then begin
      dx := round(cos(cPhase) * (rx div 2));
      Canvas.Brush.Color := FBackgroundColor;
      Canvas.Pen.Color := FBackgroundColor;
      Canvas.Ellipse(x0 + rx div 2 - dx, y0, x0 + rx div 2 + dx, y0 + ry);
      Canvas.Rectangle(x0, y0, x0 + rx div 2, y0 + ry);
    end
    else
      if cPhase < pi
        then begin
          dx := round(cos(cPhase) * (rx div 2));
          Canvas.Brush.Color := FBackgroundColor;
          Canvas.Pen.Color := FBackgroundColor;
          Canvas.Rectangle(x0, y0, x0 + rx div 2, y0 + ry);
          Canvas.Brush.Color := FMoonColor;
          Canvas.Pen.Color := FMoonColor;
          Canvas.Ellipse(x0 + rx div 2 - dx, y0, x0 + rx div 2 + dx, y0 + ry);
        end
        else
          if cPhase < threeAndAHalfPi
            then begin
              dx := round(cos(cPhase) * (rx div 2));
              Canvas.Brush.Color := FBackgroundColor;
              Canvas.Pen.Color := FBackgroundColor;
              Canvas.Rectangle(x0 + rx div 2 + 1, y0, x0 + rx, y0 + ry);
              Canvas.Brush.Color := FMoonColor;
              Canvas.Pen.Color := FMoonColor;
              Canvas.Ellipse(x0 + rx div 2 - dx, y0, x0 + rx div 2 + dx, y0 + ry);
            end
            else
              if cPhase < twoPiS
                then begin
                  dx := round(cos(cPhase) * (rx div 2));
                  Canvas.Brush.Color := FBackgroundColor;
                  Canvas.Pen.Color := FBackgroundColor;
                  Canvas.Rectangle(x0 + rx div 2 + 1, y0, x0 + rx, y0 + ry);
                  Canvas.Ellipse(x0 + rx div 2 - dx, y0, x0 + rx div 2 + dx, y0 + ry);
                end;
end;

function TMoon.CalcPhase;
const
  SynodicMoon = 2.78 + 60 * (48 + 60 * (12 + 24 * 29));    //   in seconds
  NewMoon = 60 * (33 + 60 * (7 + 24 * 46));      //  15th Feb 99 @ 07:33AM
  aYear = 46.1 + 60 * (48 + 60 * (5 + 24 * 365));     // 365d 5h 48m 46.1s
  DazeTo : array [1..12] of integer = (
    0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334
  );
var
  year, mon, day, hour, min, sec, mSec : word;
  cTime, cPhase : extended;
  CustomDateTime : TDateTime;
begin
  if FMoonDisplayType = mdtUseSystemTime
    then begin
      DecodeDate(Now, year, mon, day);
      DecodeTime(Now, hour, min, sec, mSec);
      FCustomTime := DateTimeToStr(Now);
    end
    else
      if FMoonDisplayType = mdtUseCustomTime
        then begin
          CustomDateTime := StrToDateTime(FCustomTime);
          DecodeDate(CustomDateTime, year, mon, day);
          DecodeTime(CustomDateTime, hour, min, sec, mSec);
        end
        else
          begin
            result := FPhase;
            exit;
          end;

  //   current time
  cTime := sec + mSec / 100 + 60 * (min + 60 * (hour + 24 * (day + DazeTo[mon]))) + (year - 1999) * aYear;

  cTime := cTime - NewMoon;
  cTime := cTime - trunc(cTime / SynodicMoon) * SynodicMoon;
  cPhase := twoPiS * cTime / SynodicMoon;    // current phase

  if cPhase > 0
    then FPhase := cPhase
    else FPhase := twoPiS + cPhase;

  Result := FPhase;
end;

end.

